<?php
namespace app\cms\controller;

use app\common\controller\Front;

class Filter extends Front
{
    private $items = [];
    
    //继承上级
    public function _initialize()
    {
        //请求验证
        if( !cmsRequestCheck($this->request->ip(),$this->request->header('user-agent')) ){
            $this->error(lang('cms_error_rest'), 'cms/index/index');
        }
        //请求过滤
        $this->request->filter('trim,strip_tags,htmlspecialchars');
        //继承上级
        parent::_initialize();
    }
    
    //按条件筛选页
    public function index()
    {
        //查询数据
        $info = $this->detail();
        //变量赋值
        $this->assign($info);
        //数据列表
        $this->assign($this->items);
        //加载模板
        return $this->fetch();
    }

    //空操作
    public function _empty($action='')
    {
        //扩展开关
        if(!config('cms.action_name')){
            $this->error(lang('cms_error_params'));
        }
        //允许的操作名
        if(!in_array($action,explode(',',config('cms.action_name')))){
            $this->error(lang('cms_error_params'));
        }
        //查询数据
        $term = $this->detail();
        //变量赋值
        $this->assign($term);
        //数据列表
        $this->assign($this->items);
        //默认加载
        return $this->fetch();
    }
    
    //公共操作
    private function detail()
    {
        //过滤URL空值
        $this->query        = DcArrayEmpty($this->query);
        //扩展字段列表
        $fieldsMeta         = DcArrayFilter($this->query, cmsMetaKeys('detail', NULL));
        //筛选初始参数
        $info = [];
        $info['term_id']    = $this->query['term_id'];
        $info['info_type']  = $this->cmsType($this->query['info_type']);
        $info['sortName']   = $this->sortName($this->query['sortName'], config('cms.sort_filter'));
        $info['sortOrder']  = $this->sortOrder($this->query['sortOrder'], 'desc');
        $info['pageSize']   = cmsPageSize(DcEmpty($this->query['pageSize'],config('cms.limit_filter')));
        if(config('cms.total_filter') && config('cms.page_start')){
            $info['pageStart'] = $this->query['pageStart'];
            $pageKey = 'pageStart';
        }else{
            $info['pageNumber'] = $this->site['page'];
            $pageKey = 'pageNumber';
        }
        //定义筛选变量列表
        $info['pageFilter'] = DcArrayEmpty(DcArrayArgs($fieldsMeta,$info));
        //定义分页PATH变量
        $info['pagePath']   = DcUrl('cms/filter/'.$this->site['action'],array_merge($info['pageFilter'],[$pageKey=>'[PAGE]']));
        //定义重置条件变量
        $info['pageReset']  = DcUrl('cms/filter/'.$this->site['action'],[
            'sortName'   => 'info_id',
            'sortOrder'  => 'desc',
        ]);
        //页码列表
        $info['pageSizes'] = [
            '10'  => 10,
            '20'  => 20,
            '30'  => 30,
            '50'  => 50,
            '100' => 100,
        ];
        //排序字段列表
        $info['sortNames'] = [
            'info_id'          => 'ID',
            'info_views'       => lang('cms_views'),
            'info_update_time' => lang('cms_update_time'),
            'info_order'       => lang('cms_order'),
            'cms_up'           => lang('cms_up'),
        ];
        //排序方式列表
        $info['sortOrders'] = [
            'desc' => lang('cms_order_desc'),
            'asc'  => lang('cms_order_asc'),
        ];
        //筛选列表
        $info['types'] = cmsTypeOption();
        //分类列表
        $terms = cmsCategorySelect([
            'cache'    => true,
            'status'   => 'normal',
            'result'   => 'array',
            'field'    => 'term_id,term_slug,term_name,term_action',
            'with'     => false,
            'limit'    => 0,
            'page'     => 0,
            'sort'     => 'term_count desc,term_id',
            'order'    => 'desc',
            'action'   => $this->site['action'],
        ]);
        foreach($terms as $key=>$value){
            $info['termIds'][$value['term_id']] = $value['term_name'];
            $info['termSlugs'][$value['term_slug']] = $value['term_name'];
        }
        unset($terms);
        //地址栏参数
        $info['query'] = $this->query;
        
        //数据查询条件
        $args = [];
        $args['cache']      = true;
        $args['start']      = $this->query['pageStart'];
        $args['status']     = 'normal';
        $args['action']     = $this->site['action'];
        $args['controll']   = 'detail';
        $args['module']     = 'cms';
        $args['with']       = 'info_meta,term';
        $args['field']      = config('cms.field_select');
        $args['simple']     = config('cms.total_filter');
        $args['sort']       = $info['sortName'];
        $args['order']      = $info['sortOrder'];
        $args['paginate']   = [
            'list_rows' => $info['pageSize'],
            'page'      => $info['pageNumber'],
            'path'      => $info['pagePath'],
        ];
        //按META字段条件筛选
        $args['meta_query'] = cmsMetaQuery($this->query);
        //按META字段排序
        if( !in_array($args['sort'],['info_id','info_order','info_views','info_hits','info_create_time','info_update_time']) ){
            $args['meta_key'] = $args['sort'];
            $args['sort']     = 'meta_value_num';
        }
        //文章内型限制
        if($info['info_type']){
            $args['type'] = ['eq',$info['info_type']];
        }
        //分类ID限制
        if($info['term_id']){
            $args['term_id'] = ['eq',intval($info['term_id'])];
        }
        //数据查询
        $this->items = cmsSelect($args);
        if(config('cms.total_filter') && config('cms.page_start')){
            $this->items['page_list'] = cmsPageList($this->items['next_item'][0]['info_id'],$info['pagePath']);
        }
        //返回结果
        return $info;
    }
    
    //文章形式
    private function cmsType($infoType='')
    {
        if( in_array($infoType,array_keys(cmsTypeOption())) ){
            return $infoType;
        }
        return '';
    }
    
    //排序字段
    private function sortName($sortName='',$sortDefault='info_update_time')
    {
        if( in_array($sortName,['cms_up','cms_down','info_id','info_order','info_views','info_hits','info_create_time','info_update_time']) ){
            return $sortName;
        }
        return $sortDefault;
    }
    
    //排序方式
    private function sortOrder($sortOrder='',$sortDefault='desc')
    {
        if( in_array($sortOrder,['desc','asc']) ){
            return $sortOrder;
        }
        return $sortDefault;
    }
}